#ifndef _dds_
#define _dds_

/* Copyright (c) 2020 Boulder Real Time Technologies, Inc. */
/* All rights reserved */

/* This software module is wholly owned by Boulder Real Time 
   Technologies, Inc. Any use of this software module without
   express written permission from Boulder Real Time Technologies,
   Inc. is prohibited. */

#include <stddef.h>

#include "swapbytes.h"

#define	DDS_FILE 		1
#define	DDS_HEADER		2
#define	DDS_HEADER10	3
#define	DDS_PIXELFORMAT	4

#define	DDS_TYPE_DWORD			1
#define	DDS_TYPE_HEADER			2
#define	DDS_TYPE_HEADER10		3
#define	DDS_TYPE_DXGI_FORMAT	4
#define	DDS_TYPE_D3D10_RESOURCE_DIMENSION	5
#define	DDS_TYPE_PIXELFORMAT	6
#define	DDS_TYPE_HEADER_DWFLAGS		7
#define	DDS_TYPE_PIXELFORMAT_DWFLAGS 8
#define	DDS_TYPE_CHAR4 9
#define	DDS_TYPE_DDS_CAPS 10

#define	DDSD_CAPS				(0x1)
#define	DDSD_HEIGHT				(0x2)
#define	DDSD_WIDTH				(0x4)
#define	DDSD_PITCH				(0x8)
#define	DDSD_PIXELFORMAT		(0x1000)
#define	DDSD_MIPMAPCOUNT		(0x20000)
#define	DDSD_LINEARSIZE			(0x80000)
#define	DDSD_DEPTH				(0x800000)

#define	DDPF_ALPHAPIXELS		(0x1)
#define	DDPF_ALPHA				(0x2)
#define	DDPF_FOURCC				(0x4)
#define	DDPF_RGB				(0x40)
#define	DDPF_YUV				(0x200)
#define	DDPF_LUMINANCE			(0x20000)

#define	DDSCAPS_COMPLEX		(0x8)
#define	DDSCAPS_MIPMAP		(0x400000)
#define	DDSCAPS_TEXTURE		(0x1000)

typedef enum {
  DXGI_FORMAT_UNKNOWN,
  DXGI_FORMAT_R32G32B32A32_TYPELESS,
  DXGI_FORMAT_R32G32B32A32_FLOAT,
  DXGI_FORMAT_R32G32B32A32_UINT,
  DXGI_FORMAT_R32G32B32A32_SINT,
  DXGI_FORMAT_R32G32B32_TYPELESS,
  DXGI_FORMAT_R32G32B32_FLOAT,
  DXGI_FORMAT_R32G32B32_UINT,
  DXGI_FORMAT_R32G32B32_SINT,
  DXGI_FORMAT_R16G16B16A16_TYPELESS,
  DXGI_FORMAT_R16G16B16A16_FLOAT,
  DXGI_FORMAT_R16G16B16A16_UNORM,
  DXGI_FORMAT_R16G16B16A16_UINT,
  DXGI_FORMAT_R16G16B16A16_SNORM,
  DXGI_FORMAT_R16G16B16A16_SINT,
  DXGI_FORMAT_R32G32_TYPELESS,
  DXGI_FORMAT_R32G32_FLOAT,
  DXGI_FORMAT_R32G32_UINT,
  DXGI_FORMAT_R32G32_SINT,
  DXGI_FORMAT_R32G8X24_TYPELESS,
  DXGI_FORMAT_D32_FLOAT_S8X24_UINT,
  DXGI_FORMAT_R32_FLOAT_X8X24_TYPELESS,
  DXGI_FORMAT_X32_TYPELESS_G8X24_UINT,
  DXGI_FORMAT_R10G10B10A2_TYPELESS,
  DXGI_FORMAT_R10G10B10A2_UNORM,
  DXGI_FORMAT_R10G10B10A2_UINT,
  DXGI_FORMAT_R11G11B10_FLOAT,
  DXGI_FORMAT_R8G8B8A8_TYPELESS,
  DXGI_FORMAT_R8G8B8A8_UNORM,
  DXGI_FORMAT_R8G8B8A8_UNORM_SRGB,
  DXGI_FORMAT_R8G8B8A8_UINT,
  DXGI_FORMAT_R8G8B8A8_SNORM,
  DXGI_FORMAT_R8G8B8A8_SINT,
  DXGI_FORMAT_R16G16_TYPELESS,
  DXGI_FORMAT_R16G16_FLOAT,
  DXGI_FORMAT_R16G16_UNORM,
  DXGI_FORMAT_R16G16_UINT,
  DXGI_FORMAT_R16G16_SNORM,
  DXGI_FORMAT_R16G16_SINT,
  DXGI_FORMAT_R32_TYPELESS,
  DXGI_FORMAT_D32_FLOAT,
  DXGI_FORMAT_R32_FLOAT,
  DXGI_FORMAT_R32_UINT,
  DXGI_FORMAT_R32_SINT,
  DXGI_FORMAT_R24G8_TYPELESS,
  DXGI_FORMAT_D24_UNORM_S8_UINT,
  DXGI_FORMAT_R24_UNORM_X8_TYPELESS,
  DXGI_FORMAT_X24_TYPELESS_G8_UINT,
  DXGI_FORMAT_R8G8_TYPELESS,
  DXGI_FORMAT_R8G8_UNORM,
  DXGI_FORMAT_R8G8_UINT,
  DXGI_FORMAT_R8G8_SNORM,
  DXGI_FORMAT_R8G8_SINT,
  DXGI_FORMAT_R16_TYPELESS,
  DXGI_FORMAT_R16_FLOAT,
  DXGI_FORMAT_D16_UNORM,
  DXGI_FORMAT_R16_UNORM,
  DXGI_FORMAT_R16_UINT,
  DXGI_FORMAT_R16_SNORM,
  DXGI_FORMAT_R16_SINT,
  DXGI_FORMAT_R8_TYPELESS,
  DXGI_FORMAT_R8_UNORM,
  DXGI_FORMAT_R8_UINT,
  DXGI_FORMAT_R8_SNORM,
  DXGI_FORMAT_R8_SINT,
  DXGI_FORMAT_A8_UNORM,
  DXGI_FORMAT_R1_UNORM,
  DXGI_FORMAT_R9G9B9E5_SHAREDEXP,
  DXGI_FORMAT_R8G8_B8G8_UNORM,
  DXGI_FORMAT_G8R8_G8B8_UNORM,
  DXGI_FORMAT_BC1_TYPELESS,
  DXGI_FORMAT_BC1_UNORM,
  DXGI_FORMAT_BC1_UNORM_SRGB,
  DXGI_FORMAT_BC2_TYPELESS,
  DXGI_FORMAT_BC2_UNORM,
  DXGI_FORMAT_BC2_UNORM_SRGB,
  DXGI_FORMAT_BC3_TYPELESS,
  DXGI_FORMAT_BC3_UNORM,
  DXGI_FORMAT_BC3_UNORM_SRGB,
  DXGI_FORMAT_BC4_TYPELESS,
  DXGI_FORMAT_BC4_UNORM,
  DXGI_FORMAT_BC4_SNORM,
  DXGI_FORMAT_BC5_TYPELESS,
  DXGI_FORMAT_BC5_UNORM,
  DXGI_FORMAT_BC5_SNORM,
  DXGI_FORMAT_B5G6R5_UNORM,
  DXGI_FORMAT_B5G5R5A1_UNORM,
  DXGI_FORMAT_B8G8R8A8_UNORM,
  DXGI_FORMAT_B8G8R8X8_UNORM,
  DXGI_FORMAT_R10G10B10_XR_BIAS_A2_UNORM,
  DXGI_FORMAT_B8G8R8A8_TYPELESS,
  DXGI_FORMAT_B8G8R8A8_UNORM_SRGB,
  DXGI_FORMAT_B8G8R8X8_TYPELESS,
  DXGI_FORMAT_B8G8R8X8_UNORM_SRGB,
  DXGI_FORMAT_BC6H_TYPELESS,
  DXGI_FORMAT_BC6H_UF16,
  DXGI_FORMAT_BC6H_SF16,
  DXGI_FORMAT_BC7_TYPELESS,
  DXGI_FORMAT_BC7_UNORM,
  DXGI_FORMAT_BC7_UNORM_SRGB,
  DXGI_FORMAT_AYUV,
  DXGI_FORMAT_Y410,
  DXGI_FORMAT_Y416,
  DXGI_FORMAT_NV12,
  DXGI_FORMAT_P010,
  DXGI_FORMAT_P016,
  DXGI_FORMAT_420_OPAQUE,
  DXGI_FORMAT_YUY2,
  DXGI_FORMAT_Y210,
  DXGI_FORMAT_Y216,
  DXGI_FORMAT_NV11,
  DXGI_FORMAT_AI44,
  DXGI_FORMAT_IA44,
  DXGI_FORMAT_P8,
  DXGI_FORMAT_A8P8,
  DXGI_FORMAT_B4G4R4A4_UNORM,
  DXGI_FORMAT_P208,
  DXGI_FORMAT_V208,
  DXGI_FORMAT_V408,
  DXGI_FORMAT_FORCE_UINT
} DXGI_FORMAT;

typedef enum {
  D3D10_RESOURCE_DIMENSION_UNKNOWN,
  D3D10_RESOURCE_DIMENSION_BUFFER,
  D3D10_RESOURCE_DIMENSION_TEXTURE1D,
  D3D10_RESOURCE_DIMENSION_TEXTURE2D,
  D3D10_RESOURCE_DIMENSION_TEXTURE3D
} D3D10_RESOURCE_DIMENSION;

/* dds pixel format structure */
typedef struct {
  DXGI_FORMAT dxgiFormat;
  D3D10_RESOURCE_DIMENSION resourceDimension;
  unsigned long int miscFlag;
  unsigned long int arraySize;
  unsigned long int miscFlag2;
} DDSHEADER10Str;

/* dds pixel format structure */
typedef struct { 
  unsigned long int dwSize;
  unsigned long int dwFlags;
  unsigned long int dwFourCC;
  unsigned long int dwRGBBitCount;
  unsigned long int dwRBitMask;
  unsigned long int dwGBitMask;
  unsigned long int dwBBitMask;
  unsigned long int dwABitMask;
} DDSPIXELFORMATStr;

/* dds file header structure */
typedef struct { 
	unsigned long int dwSize;
	unsigned long int dwFlags;
	unsigned long int dwHeight;
	unsigned long int dwWidth;
	unsigned long int dwPitchOrLinearSize;
	unsigned long int dwDepth;
	unsigned long int dwMipMapCount;
	unsigned long int dwReserved[11];
	DDSPIXELFORMATStr ddspf;
	unsigned long int dwCaps;
	unsigned long int dwCaps2;
	unsigned long int dwCaps3;
	unsigned long int dwCaps4;
	unsigned long int dwReserved2;
} DDSHEADERStr;

/* dds file structure */
typedef struct dds_file_str_ { 
	unsigned long int dwMagic;
	DDSHEADERStr header;
	DDSHEADER10Str header10;
	Tbl *argb_mipmaps;
} DDSFILEStr;

#ifdef  __cplusplus
extern "C" {
#endif

extern int dds_decode_file (DDSFILEStr **fstr, int mipmap_index, unsigned char *stream0, unsigned char **stream);
extern void dds_print_file (char *prefix, void *dstruc);

#ifdef  __cplusplus
}
#endif

#endif