#ifndef _esri_
#define _esri_

/* Copyright (c) 2002 Boulder Real Time Technologies, Inc. */
/* All rights reserved */

/* This software module is wholly owned by Boulder Real Time 
   Technologies, Inc. Any use of this software module without
   express written permission from Boulder Real Time Technologies,
   Inc. is prohibited. */

#include <stdio.h>
#include <unistd.h>
#include <strings.h>

#include "db.h"
#include "stock.h"
#include "coords.h"
#include "arrays.h"

/* shape type codes */
#define ESRI_SHP_FILE		63	/* esri shapefile */
#define ESRI_SHP_RECORD		62	/* esri shapefile */
#define ESRI_SHP_POINT		61	/* esri shapefile point */
#define ESRI_SHP_BOX		60	/* esri shapefile box */
#define ESRI_SHP_FILEHDR	59	/* esri shapefile header */
#define ESRI_SHP_RECHDR		58	/* esri shapefile record header */
#define ESRI_SHP_RECNULL	0	/* esri shapefile Null shape record */
#define ESRI_SHP_RECPOINT	1	/* esri shapefile Point shape record */
#define ESRI_SHP_RECMPOINT	8	/* esri shapefile MultiPoint shape record */
#define ESRI_SHP_RECPOLYL	3	/* esri shapefile PolyLine shape record */
#define ESRI_SHP_RECPOLYG	5	/* esri shapefile Polygon shape record */
#define ESRI_SHP_RECPOINTM	21	/* esri shapefile Measured Point shape record */
#define ESRI_SHP_RECMPOINTM	28	/* esri shapefile Measured MultiPoint shape record */
#define ESRI_SHP_RECPOLYLM	23	/* esri shapefile Measured PolyLine shape record */
#define ESRI_SHP_RECPOLYGM	25	/* esri shapefile Measured Polygon shape record */
#define ESRI_SHP_RECPOINTZ	11	/* esri shapefile PointZ shape record */
#define ESRI_SHP_RECMPOINTZ	18	/* esri shapefile MultiPointZ shape record */
#define ESRI_SHP_RECPOLYLZ	13	/* esri shapefile PolyLineZ shape record */
#define ESRI_SHP_RECPOLYGZ	15	/* esri shapefile PolygonZ shape record */
#define ESRI_SHP_RECMULTIPA	31	/* esri shapefile MultiPatch shape record */
#define ESRI_DBASE_FILE		101	/* esri dbase file */
#define ESRI_DBASE_FILEHDR5	102	/* esri dbase file header level 5 */
#define ESRI_DBASE_FILEHDR7	103	/* esri dbase file header level 7 */
#define ESRI_DBASE_FIELDDESCR5 104	/* esri dbase field descriptor level 5 */
#define ESRI_DBASE_FIELDDESCR7 105	/* esri dbase field descriptor level 7 */
#define ESRI_DBASE_FIELDPROPS 106	/* esri dbase field properties level 5 */

/* max sizes */

/* return codes */

/* esri structure definitions */

/* esri shapefile point structure */
typedef struct esri_shppoint_str_ { 
	double x;
	double y;
} ESRISHPPOINTStr;

/* esri shapefile box structure */
typedef struct esri_shpbox_str_ { 
	double xmin;
	double ymin;
	double xmax;
	double ymax;
} ESRISHPBOXStr;

/* esri shapefile header structure */
typedef struct esri_shpfilehdr_str_ { 
	int magic;			/* should be 9994 */
	int unused1;
	int unused2;
	int unused3;
	int unused4;
	int unused5;
	int length;
	int version;			/* should be 1000 */
	int shape_type;
	double xmin;
	double ymin;
	double xmax;
	double ymax;
	double zmin;
	double zmax;
	double mmin;
	double mmax;
} ESRISHPFILEHDRStr;

/* esri shapefile structure */
typedef struct esri_shp_str_ { 
	ESRISHPFILEHDRStr hdr;
	Tbl *records;
} ESRISHPStr;

/* esri shapefile record header structure */
typedef struct esri_shprechdr_str_ { 
	int number;
	int length;
} ESRISHPRECHDRStr;

/* esri shapefile record structure */
typedef struct esri_shprec_str_ { 
	unsigned long offset;
	int shape_type;
	ESRISHPRECHDRStr hdr;
	void *record;
} ESRISHPRECStr;

/* esri shapefile Null record structure */
typedef struct esri_shprecnull_str_ { 
	int shape_type;
} ESRISHPRECNULLStr;

/* esri shapefile Point record structure */
typedef struct esri_shprecpoint_str_ { 
	int shape_type;
	double x;
	double y;
} ESRISHPRECPOINTStr;

/* esri shapefile MultiPoint record structure */
typedef struct esri_shprecmpoint_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numpoints;
	ESRISHPPOINTStr *points;
} ESRISHPRECMPOINTStr;

/* esri shapefile PolyLine record structure */
typedef struct esri_shprecpolyl_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numparts;
	int numpoints;
	int *parts;
	ESRISHPPOINTStr *points;
} ESRISHPRECPOLYLStr;

/* esri shapefile Polygon record structure */
typedef struct esri_shprecpolyg_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numparts;
	int numpoints;
	int *parts;
	ESRISHPPOINTStr *points;
} ESRISHPRECPOLYGStr;

/* esri shapefile Measured Point record structure */
typedef struct esri_shprecpointm_str_ { 
	int shape_type;
	double x;
	double y;
	double m;
} ESRISHPRECPOINTMStr;

/* esri shapefile Measured MultiPoint record structure */
typedef struct esri_shprecmpointm_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numpoints;
	ESRISHPPOINTStr *points;
	double mmin;
	double mmax;
	double *marray;
} ESRISHPRECMPOINTMStr;

/* esri shapefile Measured PolyLine record structure */
typedef struct esri_shprecpolylm_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numparts;
	int numpoints;
	int *parts;
	ESRISHPPOINTStr *points;
	double mmin;
	double mmax;
	double *marray;
} ESRISHPRECPOLYLMStr;

/* esri shapefile Measured Polygon record structure */
typedef struct esri_shprecpolygm_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numparts;
	int numpoints;
	int *parts;
	ESRISHPPOINTStr *points;
	double mmin;
	double mmax;
	double *marray;
} ESRISHPRECPOLYGMStr;

/* esri shapefile PointZ record structure */
typedef struct esri_shprecpointz_str_ { 
	int shape_type;
	double x;
	double y;
	double z;
	double m;
} ESRISHPRECPOINTZStr;

/* esri shapefile MultiPointZ record structure */
typedef struct esri_shprecmpointz_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numpoints;
	ESRISHPPOINTStr *points;
	double zmin;
	double zmax;
	double *zarray;
	double mmin;
	double mmax;
	double *marray;
} ESRISHPRECMPOINTZStr;

/* esri shapefile PolyLineZ record structure */
typedef struct esri_shprecpolylz_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numparts;
	int numpoints;
	int *parts;
	ESRISHPPOINTStr *points;
	double zmin;
	double zmax;
	double *zarray;
	double mmin;
	double mmax;
	double *marray;
} ESRISHPRECPOLYLZStr;

/* esri shapefile PolygonZ record structure */
typedef struct esri_shprecpolygz_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numparts;
	int numpoints;
	int *parts;
	ESRISHPPOINTStr *points;
	double zmin;
	double zmax;
	double *zarray;
	double mmin;
	double mmax;
	double *marray;
} ESRISHPRECPOLYGZStr;

/* esri shapefile MultiPatch record structure */
typedef struct esri_shprecmultipa_str_ { 
	int shape_type;
	ESRISHPBOXStr bbox;
	int numparts;
	int numpoints;
	int *parts;
	int *parttypes;
	ESRISHPPOINTStr *points;
	double zmin;
	double zmax;
	double *zarray;
	double mmin;
	double mmax;
	double *marray;
} ESRISHPRECMULTIPAStr;

/* esri dbase file header structure */
typedef struct esri_dbasehdr_str_ { 
	unsigned char flags;
	unsigned char date_update[3];
	int numrecs;
	short int numbyteshdr;
	short int numbytesrec;
	short int unused1;
	unsigned char incomplete;
	unsigned char encrypted;
	int unused2;
	int unused3;
	int unused4;
	unsigned char mdx;
	unsigned char langid;
	short int unused5;
	char langdr[32];
	int unused6;
} ESRIDBASEHDRStr;

/* esri dbase file header structure */
typedef struct esri_dbasefielddescr_str_ { 
	char name[12];
	char type;
	unsigned char length;
	unsigned char count;
	short int unused1;
	unsigned char mdx;
	short int unused2;
	int next;
	int unused3;
	int unused4;
	short int workid;
	unsigned char example;
	unsigned char unused5;
} ESRIDBASEFIELDDESCRStr;

/* esri dbase field properties structure */
typedef struct esri_dbasefieldprops_str_ { 
	short int numstand;
	short int startstand;
	short int numcust;
	short int startcust;
	short int numref;
	short int startref;
	short int startdata;
	short int sizestr;
	Tbl *standard_properties;
	Tbl *custom_properties;
	Tbl *referential_properties;
} ESRIDBASEFIELDPROPSStr;

/* esri dbase field standard properties structure */
typedef struct esri_dbasefieldstand_str_ { 
	short int gnumber;
	short int offset_field;
	unsigned char which;
	unsigned char type;
	unsigned char constraint;
	int unused1;
	short int offset_data;
	short int size;
} ESRIDBASEFIELDSTANDStr;

/* esri dbase field custom properties structure */
typedef struct esri_dbasefieldcust_str_ { 
	short int gnumber;
	short int offset_field;
	unsigned char type;
	unsigned char unused1;
	short int offset_name;
	short int size_name;
	short int offset_prop;
	short int size_prop;
} ESRIDBASEFIELDCUSTStr;

/* esri dbase field referential properties structure */
typedef struct esri_dbasefieldref_str_ { 
	unsigned char lineage;
	short int sequence;
	short int offset_ref;
	short int size_ref;
	short int offset_for;
	short int size_for;
	unsigned char update;
	short int numfields;
	short int offset_tag;
	short int size_tag;
	short int offset_ftag;
	short int size_ftag;
} ESRIDBASEFIELDREFStr;

/* esri dbase file structure */
typedef struct esri_dbase_str_ { 
	int level;
	ESRIDBASEHDRStr hdr;
	Tbl *field_descriptors;
	ESRIDBASEFIELDPROPSStr field_properties;
	unsigned char *records;
} ESRIDBASEStr;

#ifdef  __cplusplus
extern "C" {
#endif

extern int esri_strdefs_init ();
extern int esri_unstuff_stream (int str_type, void **estruc, unsigned char *stream0, unsigned char **stream);
extern int esri_decode_shapefile (void *estruc, unsigned char *stream0, unsigned char **stream);
extern void esri_print_shapefile (char *prefix, void *estruc);
extern void esri_print_shapefile_record (char *prefix, void *estruc, int recno);
extern int esri_decode_dbase_file (void *dstruc, unsigned char *stream0, unsigned char **stream);
extern int esri_get_dbase_fields (ESRIDBASEStr *dbstr, int recno, Arr **fields);
extern int esri_get_db_shp_record (Dbptr db, ESRISHPRECStr *record);
extern void esri_print_dbase_file (char *prefix, void *estruc);
extern char *esri_get_shapetype (int shape_type);

#ifdef  __cplusplus
}
#endif

#endif

