#ifndef _netcdf_
#define _netcdf_

/* Copyright (c) 2002 Boulder Real Time Technologies, Inc. */
/* All rights reserved */

/* This software module is wholly owned by Boulder Real Time 
   Technologies, Inc. Any use of this software module without
   express written permission from Boulder Real Time Technologies,
   Inc. is prohibited. */

#include "stock.h"

/* codes */
#define NETCDF_CODE_BYTE	1	/* data is array of 8-bit signed integers */
#define NETCDF_CODE_CHAR	2	/* data is array of characters */
#define NETCDF_CODE_SHORT	3	/* data is array of 16-bit signed integers */
#define NETCDF_CODE_INT		4	/* data is array of 32-bit signed integers */
#define NETCDF_CODE_FLOAT	5	/* data is array of IEEE single precision floats */
#define NETCDF_CODE_DOUBLE	6	/* data is array of IEEE double precision floats */
#define NETCDF_CODE_DIMENSION	10	/* code to identify dimensions section */
#define NETCDF_CODE_VARIABLE	11	/* code to identify variables section */
#define NETCDF_CODE_ATTRIBUTE	12	/* code to identify attributes section */

/* max sizes */

/* return codes */

/* netcdf structure definitions */

typedef struct netcdf_values_ {	/* an array of netcdf values */
	int type;			/* type (e.g. NETCDF_CODE_INT, etc.) */
	int n;				/* number of values */
	void *values;			/* array of values */
}NETCDFValues;

typedef struct netcdf_dim_ {	/* "dimension" entry */
	char *name;			/* dimension name */
	int value;			/* dimension value */
} NETCDFDim;

typedef struct netcdf_attr_ {	/* "attribute" entry */
	char *name;			/* attribute name */
	NETCDFValues values;		/* attribute values */
} NETCDFAttr;

typedef struct netcdf_var_ {	/* "variable" entry */
	char *name;			/* variable name */
	int ndims;			/* number of dimensions */
	NETCDFDim **dims;		/* pointers to the dimension definitions */
	int nattrs;			/* number in attributes array */
	NETCDFAttr *attrs;		/* variable "attributes" array */
	int type;			/* variable type (e.g. NETCDF_CODE_INT, etc.) */
	int vsize;			/* variable size */
	int begin;			/* variable start location (file byte offset) */
	int fixed;			/* If set, then this is a fixed size, or non-record variable */
} NETCDFVar;

typedef struct netcdf_hdr_ {	/* header */
	unsigned char *stream;		/* File stream pointer */
	int numrecs;			/* number of data "records" */
	NETCDFDim *dims;		/* "dimensions" array */
	int dims_n;			/* number in dimensions array */
	NETCDFAttr *attrs;		/* Global "attributes" array */
	int attrs_n;			/* number in attributes array */
	NETCDFVar *vars;		/* "variables" array */
	int vars_n;			/* number in variables array */
	Arr *vars_array;		/* Associative array that indexes into vars structures */
} NETCDFHdr;

typedef struct netcdf_ {	/* maps to a single cdf file */
	NETCDFHdr hdr;			/* header */
} NETCDF;

#ifdef  __cplusplus
extern "C" {
#endif

extern int netcdf_unstuff_header (unsigned char **stream, NETCDFHdr **hdr);
extern int netcdf_print_header (NETCDFHdr *hdr, FILE *file);

#ifdef  __cplusplus
}
#endif

#endif
